/*  File: analyser-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   local headers  */
#include "analyzer-jets.h"


void UserJets::initfunc()
{
  /*   Createing histograms to calculate the one jet cross sections   */
  unsigned int nbins = 48;
  double pTmin = 250.0_GeV, pTmax = 5050.0_GeV;
  auto bins = spacing<hist1d>::linear(nbins, pTmin, pTmax);
  
  /*   cross sections at the hard level   */
  Plain::phys(1, "One jet inclusive cross section at the hard interaction", bins);

  /*   cross sections after shower    */
  Ratio::phys(1, "One jet inclusive cross section LO pdf", bins);
  Ratio::phys(2, "One jet inclusive cross section NLO pdf", bins);
}



void UserJets::userfunc(const shower_history<hhc>& history)
{
  constexpr double tonb = 389379.338;

  /*   the shower stage after the shower evolution   */
  for(auto it = history.cbegin(); it != history.cend(); ++it)
  {
    /*   showered and hard stages    */
    auto& showered = *it;
    auto& hard = it.hard();
    
    /*   showered and hard events   */
    auto& p = it->state;
    auto& q = hard.state;
    
    
    //   One jet inclusive cross section after showering
    //  -------------------------------------------------
    std::array<double, 3u> w1, w2;
    
    w1[2] = showered.weights.unitary.main*color_weight(showered)*tonb;
    w1[1] = w1[2]*showered.weights.threshold.main;
    w1[0] = w1[1]*showered.weights.threshold.delta;
    
    w2[2] = w1[2]*showered.weights.unitary.pdf_ratio;
    w2[1] = w1[1]*showered.weights.unitary.pdf_ratio;
    w2[0] = w1[0]*showered.weights.unitary.pdf_ratio;
    
    
    /*   Converting deductor event to FastJet format   */
    make_pseudojets(theparticles, p);
    
    /*    Run fastjet algorithm   */
    fj::ClusterSequence clustSeq(theparticles, jetDef);
    
    /*    Extract inclusive jets sorted by pT (above pTjetmin)   */
    double pTjetmin = 200.0_GeV;
    incJets = clustSeq.inclusive_jets(pTjetmin);
    
    /*    Fill histogram, only count jets that have |y| < 2.0   */
    for(unsigned j = 0; j < incJets.size(); j++)
      if(abs(incJets[j].pseudorapidity()) < 2.0) {
        Ratio::physfill(1, w1, dirac{}, incJets[j].perp());
        Ratio::physfill(2, w2, dirac{}, incJets[j].perp());
      }
    
    
    //   One jet inclusive cross section before showering
    //  --------------------------------------------------
    std::array<double, 2u> wh;
    
    wh[1] = hard.weights.unitary.main*color_weight(hard)*tonb;
    wh[0] = wh[1]*hard.weights.unitary.pdf_ratio;

    
    /*   Fill histogram, only count jets that have |y| < 2.0
     *   The two jets are back-to-back, since we have only two
     *   partons in the final state.
     */
    if(abs(q[1].momentum.prapidity()) < 2.0)
      Plain::physfill(1, wh, dirac{}, q[1].momentum.perp());
    
    if(abs(q[2].momentum.prapidity()) < 2.0)
      Plain::physfill(1, wh, dirac{}, q[2].momentum.perp());
  }
}

